# -*- coding: UTF-8 -*-
#
#      core.py
#      
#      Copyright 2009-2010
#      Giuseppe Penone <giuspen@gmail.com>,
#      Chris Camacho (chris_c) <chris_camacho@yahoo.com>.
#      
#      plus many thanks to  http://tronche.com/gui/x/xlib/
#                      and  http://tripie.sweb.cz/utils/wmctrl/
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#      
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#      
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#      MA 02110-1301, USA.

import gtk, gobject, gconf
import sys, ctypes, webbrowser, time
import cons, support, tilings


class InfoModel:
   """Holds the information"""
      
   def __init__(self):
      """Sets up and populates the gtk.ListStore"""
      # 0:selected, 1:window ref, 2:title, 3:pixbuf of icon, 4:selected2
      self.liststore = gtk.ListStore(gobject.TYPE_BOOLEAN,
                                     int,
                                     str,
                                     gobject.TYPE_PYOBJECT,
                                     gobject.TYPE_BOOLEAN)
      self.process_picklist = set()
      self.process_blacklist = set()
      
   def load_model(self, appletobj):
      """Populates the gtk.ListStore"""
      self.liststore.clear()
      clients = []
      support.get_property("_NET_CLIENT_LIST", glob.root, glob.XA_WINDOW)
      for i in range(0, glob.num_items.value):
         clients.append(glob.ret_pointer[i])
      rows_num = 0
      support.get_property("_NET_CURRENT_DESKTOP", glob.root, glob.XA_CARDINAL)
      if bool(glob.ret_pointer)==False:
         curr_workspace_num = -2
         print "DEBUG warning _NET_CURRENT_DESKTOP improperly set"
      else: curr_workspace_num = glob.ret_pointer[0] # the number of the current workspace
      gconf_client = gconf.client_get_default()
      for client in clients:
         support.get_property("_NET_WM_DESKTOP", client, glob.XA_CARDINAL)
         workspace_num = glob.ret_pointer[0]
         if gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == "True"\
         and workspace_num != curr_workspace_num:
            continue
         support.get_property("_NET_WM_NAME", client, glob.str_atom)
         title="???"
         if bool(glob.ret_pointer)==False:
            print "DEBUG warning _NET_WM_NAME improperly set by application"
            support.get_property("WM_NAME", client, glob.str2_atom)
            if bool(glob.ret_pointer)==False:
               print "DEBUG warning WM_NAME not set by application"
            else:
               title = ctypes.string_at(glob.ret_pointer)
               print "class "+title
         else: title = ctypes.string_at(glob.ret_pointer)
         if workspace_num == -1 and title in cons.STATIC_WINS_BLACKLIST: continue
         if gtk.gdk.window_lookup(client) == appletobj.glade.window.window: continue # we do not show our own window
         if appletobj.glade.configwindow.window != None and gtk.gdk.window_lookup(client) == appletobj.glade.configwindow.window:
            continue # nor the pref window!
         if appletobj.glade.filterdialog.window != None and gtk.gdk.window_lookup(client) == appletobj.glade.filterdialog.window:
            continue # nor the filter window!
         pxb = support.get_icon(client)
         if pxb: pxb = pxb.scale_simple(24, 24, gtk.gdk.INTERP_BILINEAR)
         support.get_property("_NET_WM_PID", client, glob.XA_CARDINAL)
         pid=0
         process_name="UNKNOWN"
         if bool(glob.ret_pointer)==False:
            print "DEBUG warning can't get PID _NET_WM_PID failed"
         else:
            pid = glob.ret_pointer[0]
            process_name = support.get_process_name(pid)
         # filter based on process name
         if process_name != "gnome-panel": # hard coded filter
            self.process_picklist.add(process_name)
            if process_name not in self.process_blacklist: # user filter
               self.liststore.append([False, client, title, pxb, False])
               rows_num += 1
      if rows_num == 2:
         iter = self.liststore.get_iter_first()
         while iter != None:
            self.liststore[iter][0] = True
            iter = self.liststore.iter_next(iter)
   
   def flag_all_rows(self):
      """Flags All Rows"""
      if glob.num_monitors > 1:
         all_rows_flagged_on_monitor_1 = True
         iter = self.liststore.get_iter_first()
         while iter != None:
            if self.liststore[iter][0] == False:
               all_rows_flagged_on_monitor_1 = False
               break
            iter = self.liststore.iter_next(iter)
      else: all_rows_flagged_on_monitor_1 = False
      iter = self.liststore.get_iter_first()
      while iter != None:
         if not all_rows_flagged_on_monitor_1:
            self.liststore[iter][0] = True
            self.liststore[iter][4] = False
         else:
            self.liststore[iter][0] = False
            self.liststore[iter][4] = True
         iter = self.liststore.iter_next(iter)
      
   def unflag_all_rows(self):
      """Unflags All Rows"""
      iter = self.liststore.get_iter_first()
      while iter != None:
         self.liststore[iter][0] = False
         self.liststore[iter][4] = False
         iter = self.liststore.iter_next(iter)
   
   def get_checked_windows_list(self):
      """Returns the list of the checked windows"""
      checked_windows_list = [[],[]]
      iter = self.liststore.get_iter_first()
      while iter != None:
         if self.liststore[iter][0] == True:
            checked_windows_list[0].append(self.liststore[iter][1])  # win id
         elif self.liststore[iter][4] == True:
            checked_windows_list[1].append(self.liststore[iter][1])  # win id
         iter = self.liststore.iter_next(iter)
      return checked_windows_list
      
   def close_checked_windows(self):
      """Closes the checked windows and removes the rows from the model"""
      iter = self.liststore.get_iter_first()
      while iter != None:
         next_iter = self.liststore.iter_next(iter)
         if self.liststore[iter][0] == True or self.liststore[iter][4] == True:
            support.client_msg(self.liststore[iter][1],"_NET_CLOSE_WINDOW",0,0,0,0,0)
            glob.x11.XSync(glob.disp, False)
            self.liststore.remove(iter)
         iter = next_iter
      
   def row_up(self, iter):
      """Row up one position"""
      prev_iter = self.get_tree_iter_prev_sibling(iter)
      if prev_iter != None: self.liststore.swap(iter, prev_iter)
      
   def row_down(self, iter):
      """Row down one position"""
      subseq_iter = self.liststore.iter_next(iter)
      if subseq_iter != None: self.liststore.swap(iter, subseq_iter)
      
   def row_delete(self, iter):
      """Row remove"""
      self.liststore.remove(iter)
      
   def get_tree_iter_prev_sibling(self, node_iter):
      """Returns the previous sibling iter or None if the given iter is the first"""
      node_path = self.liststore.get_path(node_iter)
      sibling_index = len(node_path)-1
      prev_iter = None
      while prev_iter == None and node_path[sibling_index] > 0:
         node_path_list = list(node_path)
         node_path_list[sibling_index] -= 1
         prev_path = tuple(node_path_list)
         prev_iter = self.liststore.get_iter(prev_path)
      return prev_iter
      
   def get_model(self):
      """Returns the model"""
      return self.liststore


class GladeWidgetsWrapper:
   """Handles the retrieval of glade widgets"""
   
   def __init__(self, glade_file_path, gui_instance):
      try:
         self.glade_widgets = gtk.Builder()
         self.glade_widgets.set_translation_domain(cons.APP_NAME)
         self.glade_widgets.add_from_file(glade_file_path)
         self.glade_widgets.connect_signals(gui_instance)
      except: print "Failed to load the glade file"
      
   def __getitem__(self, key):
      """Gives us the ability to do: wrapper['widget_name'].action()"""
      return self.glade_widgets.get_object(key)
      
   def __getattr__(self, attr):
      """Gives us the ability to do: wrapper.widget_name.action()"""
      new_widget = self.glade_widgets.get_object(attr)
      if new_widget is None: raise AttributeError, 'Widget %r not found' % attr
      setattr(self, attr, new_widget)
      return new_widget


class XTile:
   """The application's main window"""
   
   def __init__(self, store):
      """Instantiate the Glade Widgets Wrapper, create the view,
      retrieves and stores the information about the running desktop self.geometry"""
      # instantiate the Glade Widgets Wrapper
      self.glade = GladeWidgetsWrapper(cons.GLADE_PATH + 'x-tile.glade', self)
      glob.alive = True # this is necessary for the use as an applet
      # ui manager
      actions = gtk.ActionGroup("Actions")
      actions.add_actions(cons.get_entries(self))
      self.ui = gtk.UIManager()
      self.ui.insert_action_group(actions, 0)
      self.glade.window.add_accel_group(self.ui.get_accel_group())
      self.ui.add_ui_from_string(cons.UI_INFO)
      # menubar add
      self.glade.vbox_main.pack_start(self.ui.get_widget("/MenuBar"), False, False)
      self.glade.vbox_main.reorder_child(self.ui.get_widget("/MenuBar"), 0)
      # toolbar add
      self.glade.vbox_main.pack_start(self.ui.get_widget("/ToolBar"), False, False)
      self.glade.vbox_main.reorder_child(self.ui.get_widget("/ToolBar"), 1)
      self.ui.get_widget("/ToolBar").set_style(gtk.TOOLBAR_ICONS)
      # create a variable pointing to the instance of the InfoModel class
      self.store = store
      # create the view
      self.view = gtk.TreeView(store.get_model())
      self.view.set_headers_visible(False)
      self.renderer_checkbox = gtk.CellRendererToggle()
      self.renderer_checkbox.set_property('activatable', True)
      self.renderer_checkbox.connect('toggled', self.toggle_active, self.store.liststore)
      self.renderer_checkbox2 = gtk.CellRendererToggle()
      self.renderer_checkbox2.set_property('activatable', True)
      self.renderer_checkbox2.connect('toggled', self.toggle_active2, self.store.liststore)
      self.renderer_pixbuf = gtk.CellRendererPixbuf()
      self.renderer_text = gtk.CellRendererText()
      self.columns = [None]*4
      self.columns[0] = gtk.TreeViewColumn("Tile", self.renderer_checkbox, active=0) # active=0 <> read from column 0 of model
      self.columns[1] = gtk.TreeViewColumn("Tile", self.renderer_checkbox2, active=4) # active=4 <> read from column 4 of model
      self.columns[2] = gtk.TreeViewColumn("Logo", self.renderer_pixbuf)
      self.columns[2].set_cell_data_func(self.renderer_pixbuf, self.make_pixbuf)
      self.columns[3] = gtk.TreeViewColumn("Window Description", self.renderer_text, text=2) # text=2 <> read from column 2 of model
      for n in range(4): self.view.append_column(self.columns[n])
      if glob.num_monitors < 2: self.columns[1].set_visible(False)
      self.view.set_reorderable(True) # allow drag and drop reordering of rows
      self.no_toggling_signals = False
      self.viewselection = self.view.get_selection()
      self.glade.scrolledwindow.add(self.view)
      self.glade.filteradddialog.connect('key_press_event', self.on_key_press_filteradddialog)
      self.glade.aboutdialog.set_version(cons.VERSION)
      self.gconf_client = gconf.client_get_default()
      self.gconf_client.add_dir("/apps/x-tile/%s" % glob.screen_index, gconf.CLIENT_PRELOAD_NONE)
      self.statusbar_context_id = self.glade.statusbar.get_context_id('')
      self.combobox_country_lang_init()
   
   def combobox_country_lang_init(self):
      """Init The Programming Languages Syntax Highlighting"""
      combobox = self.glade.combobox_country_language
      self.country_lang_liststore = gtk.ListStore(str)
      combobox.set_model(self.country_lang_liststore)
      cell = gtk.CellRendererText()
      combobox.pack_start(cell, True)
      combobox.add_attribute(cell, 'text', 0)
      for country_lang in cons.AVAILABLE_LANGS:
         self.country_lang_liststore.append([country_lang])
      combobox.set_active_iter(self.get_combobox_country_lang_iter(self.gconf_client.get_string("/apps/x-tile/language")))
      self.glade.combobox_country_language.connect('changed', self.on_combobox_country_language_changed)
   
   def get_combobox_country_lang_iter(self, country_language):
      """Returns the Language iter Given the Language Name"""
      curr_iter = self.country_lang_liststore.get_iter_first()
      while curr_iter != None:
         if self.country_lang_liststore[curr_iter][0] == country_language: break
         else: curr_iter = self.country_lang_liststore.iter_next(curr_iter)
      else: return self.country_lang_liststore.get_iter_first()
      return curr_iter
   
   def on_combobox_country_language_changed(self, combobox):
      """New Country Language Choosed"""
      new_iter = self.glade.combobox_country_language.get_active_iter()
      new_lang = self.country_lang_liststore[new_iter][0]
      if new_lang != self.gconf_client.get_string("/apps/x-tile/language"):
         self.country_lang = new_lang
         support.dialog_info(_("The New Language will be Available Only After Restarting X Tile"), self.glade.window)
         self.gconf_client.set_string("/apps/x-tile/language", new_lang)
   
   def on_checkbutton_override_monitor_1_area_toggled(self, checkbutton):
      """Override Area 1 Checkbox was Toggled"""
      if checkbutton.get_active(): cons.OVERRIDE_1 = 1
      else:
         cons.OVERRIDE_1 = 0
         glob.read_monitors_areas()
         cons.X1 = glob.monitors_areas[0][0]
         cons.Y1 = glob.monitors_areas[0][1]
         cons.W1 = glob.monitors_areas[0][2]
         cons.H1 = glob.monitors_areas[0][3]
         self.glade.spinbutton_x1.set_value(cons.X1)
         self.glade.spinbutton_y1.set_value(cons.Y1)
         self.glade.spinbutton_w1.set_value(cons.W1)
         self.glade.spinbutton_h1.set_value(cons.H1)
      self.gconf_client.set_int(cons.GCONF_OVERRIDE_1 % glob.screen_index, cons.OVERRIDE_1)
      self.glade.vbox_override_area_1.set_sensitive(cons.OVERRIDE_1 == 1)
      
   def on_checkbutton_override_monitor_2_area_toggled(self, checkbutton):
      """Override Area 1 Checkbox was Toggled"""
      if checkbutton.get_active(): cons.OVERRIDE_2 = 1
      else:
         cons.OVERRIDE_2 = 0
         glob.read_monitors_areas()
         cons.X2 = glob.monitors_areas[1][0]
         cons.Y2 = glob.monitors_areas[1][1]
         cons.W2 = glob.monitors_areas[1][2]
         cons.H2 = glob.monitors_areas[1][3]
         self.glade.spinbutton_x2.set_value(cons.X2)
         self.glade.spinbutton_y2.set_value(cons.Y2)
         self.glade.spinbutton_w2.set_value(cons.W2)
         self.glade.spinbutton_h2.set_value(cons.H2)
      self.gconf_client.set_int(cons.GCONF_OVERRIDE_2 % glob.screen_index, cons.OVERRIDE_2)
      self.glade.vbox_override_area_2.set_sensitive(cons.OVERRIDE_2 == 1)
   
   def on_spinbutton_h_overlap_adj_value_changed(self, spinbutton):
      """New Horizontal Overlap Adjustment Selected"""
      cons.WIDTH_ADJ = int(spinbutton.get_value())
      self.gconf_client.set_string(cons.GCONF_WIDTH_ADJ % glob.screen_index, str(cons.WIDTH_ADJ))
      self.update_statusbar()
      
   def on_spinbutton_v_overlap_adj_value_changed(self, spinbutton):
      """New Vertical Overlap Adjustment Selected"""
      cons.HEIGHT_ADJ = int(spinbutton.get_value())
      self.gconf_client.set_string(cons.GCONF_HEIGHT_ADJ % glob.screen_index, str(cons.HEIGHT_ADJ))
      self.update_statusbar()
    
   def init_from_gconf(self):
      """Init the geometry and the spinbuttons"""
      glob.read_monitors_areas()
      if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == None:
         self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, "True")
      if self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == None:
         self.gconf_client.set_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index, "False")
      # monitor 1 handling
      if self.gconf_client.get_int(cons.GCONF_OVERRIDE_1 % glob.screen_index) == 1:
         cons.OVERRIDE_1 = 1
         cons.X1 = self.gconf_client.get_int(cons.GCONF_X1 % glob.screen_index)
         cons.Y1 = self.gconf_client.get_int(cons.GCONF_Y1 % glob.screen_index)
         cons.W1 = self.gconf_client.get_int(cons.GCONF_W1 % glob.screen_index)
         cons.H1 = self.gconf_client.get_int(cons.GCONF_H1 % glob.screen_index)
         glob.monitors_areas[0] = [cons.X1, cons.Y1, cons.W1, cons.H1]
      else:
         cons.X1 = glob.monitors_areas[0][0]
         cons.Y1 = glob.monitors_areas[0][1]
         cons.W1 = glob.monitors_areas[0][2]
         cons.H1 = glob.monitors_areas[0][3]
      # monitor 2 handling
      if glob.num_monitors > 1:
         if self.gconf_client.get_int(cons.GCONF_OVERRIDE_2 % glob.screen_index) == 1:
            cons.OVERRIDE_2 = 1
            cons.X2 = self.gconf_client.get_int(cons.GCONF_X2 % glob.screen_index)
            cons.Y2 = self.gconf_client.get_int(cons.GCONF_Y2 % glob.screen_index)
            cons.W2 = self.gconf_client.get_int(cons.GCONF_W2 % glob.screen_index)
            cons.H2 = self.gconf_client.get_int(cons.GCONF_H2 % glob.screen_index)
            glob.monitors_areas[1] = [cons.X2, cons.Y2, cons.W2, cons.H2]
         else:
            cons.X2 = glob.monitors_areas[1][0]
            cons.Y2 = glob.monitors_areas[1][1]
            cons.W2 = glob.monitors_areas[1][2]
            cons.H2 = glob.monitors_areas[1][3]
      if self.gconf_client.get_string(cons.GCONF_WIDTH_ADJ % glob.screen_index) == None:
         self.gconf_client.set_string(cons.GCONF_WIDTH_ADJ % glob.screen_index, str(cons.WIDTH_ADJ))
      else: cons.WIDTH_ADJ = int(self.gconf_client.get_string(cons.GCONF_WIDTH_ADJ % glob.screen_index))
      if self.gconf_client.get_string(cons.GCONF_HEIGHT_ADJ % glob.screen_index) == None:
         self.gconf_client.set_string(cons.GCONF_HEIGHT_ADJ % glob.screen_index, str(cons.HEIGHT_ADJ))
      else: cons.HEIGHT_ADJ = int(self.gconf_client.get_string(cons.GCONF_HEIGHT_ADJ % glob.screen_index))
      key = self.gconf_client.get_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index)
      if key not in cons.ICONS_SIZE: self.gconf_client.set_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index, 3)
      self.ui.get_widget("/ToolBar").set_property("icon-size",
            cons.ICONS_SIZE[self.gconf_client.get_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index)])
      blacklist = self.gconf_client.get_string(cons.GCONF_PROCESS_BLACKLIST % glob.screen_index)
      if blacklist not in [None, ""]:
         for element in blacklist.split():
            self.store.process_blacklist.add(element)
      self.glade.spinbutton_h_overlap_adj.set_value(cons.WIDTH_ADJ)
      self.glade.spinbutton_v_overlap_adj.set_value(cons.HEIGHT_ADJ)
      self.glade.checkbutton_override_monitor_1_area.set_active(cons.OVERRIDE_1 == 1)
      self.glade.checkbutton_override_monitor_2_area.set_active(cons.OVERRIDE_2 == 1)
      self.glade.spinbutton_x1.set_value(cons.X1)
      self.glade.spinbutton_y1.set_value(cons.Y1)
      self.glade.spinbutton_w1.set_value(cons.W1)
      self.glade.spinbutton_h1.set_value(cons.H1)
      self.glade.spinbutton_x2.set_value(cons.X2)
      self.glade.spinbutton_y2.set_value(cons.Y2)
      self.glade.spinbutton_w2.set_value(cons.W2)
      self.glade.spinbutton_h2.set_value(cons.H2)
      self.glade.vbox_override_area_1.set_sensitive(cons.OVERRIDE_1 == 1)
      self.glade.vbox_override_area_2.set_sensitive(cons.OVERRIDE_2 == 1)
      self.glade.checkbutton_override_monitor_2_area.set_sensitive(glob.num_monitors > 1)
      self.win_size_n_pos = {}
      self.win_size_n_pos['win_size'] = [self.gconf_client.get_int(cons.GCONF_WIN_SIZE_W % glob.screen_index),
                                         self.gconf_client.get_int(cons.GCONF_WIN_SIZE_H % glob.screen_index)]
      if 0 not in self.win_size_n_pos['win_size']:
         self.glade.window.resize(self.win_size_n_pos['win_size'][0], self.win_size_n_pos['win_size'][1])
   
   def window_position_restore(self):
      """Restore window size and position"""
      self.win_size_n_pos['win_position'] = [self.gconf_client.get_int(cons.GCONF_WIN_POSITION_X % glob.screen_index),
                                             self.gconf_client.get_int(cons.GCONF_WIN_POSITION_Y % glob.screen_index)]
      self.glade.window.move(self.win_size_n_pos['win_position'][0], self.win_size_n_pos['win_position'][1])
   
   def toolbar_icons_size_increase(self, *args):
      """Increase the Size of the Toolbar Icons"""
      toolbar_icon_size = self.gconf_client.get_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index)
      if toolbar_icon_size == 5:
         support.dialog_info(_("The Size of the Toolbar Icons is already at the Maximum Value"), self.glade.window)
         return
      toolbar_icon_size += 1
      self.gconf_client.set_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index, toolbar_icon_size)
      self.ui.get_widget("/ToolBar").set_property("icon-size", cons.ICONS_SIZE[toolbar_icon_size])
   
   def toolbar_icons_size_decrease(self, *args):
      """Decrease the Size of the Toolbar Icons"""
      toolbar_icon_size = self.gconf_client.get_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index)
      if toolbar_icon_size == 1:
         support.dialog_info(_("The Size of the Toolbar Icons is already at the Minimum Value"), self.glade.window)
         return
      toolbar_icon_size -= 1
      self.gconf_client.set_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index, toolbar_icon_size)
      self.ui.get_widget("/ToolBar").set_property("icon-size", cons.ICONS_SIZE[toolbar_icon_size])
   
   def update_statusbar(self):
      """Update the Statusbar after checking checkbuttons"""
      statusbar_str = _("Version %s  -  Exit After Tile: %s  -  Overlaps Adjustment: %s|%s") %\
                       (cons.VERSION,
                        cons.ON_OFF[self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index)],
                        cons.WIDTH_ADJ,
                        cons.HEIGHT_ADJ)
      self.glade.statusbar.push(self.statusbar_context_id, statusbar_str)
   
   def toggle_active(self, cell, path, model):
      """Toggles the Active state"""
      model[path][0] = not model[path][0]
      model[path][4] = False
   
   def toggle_active2(self, cell, path, model):
      """Toggles the Active state"""
      model[path][4] = not model[path][4]
      model[path][0] = False
   
   def toggle_exit_after_tile(self, *args):
      """Toggles the flag Exit After Tile"""
      if not self.no_toggling_signals:
         if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True":
            self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, "False")
         else: self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, "True")
         self.update_statusbar()
         
   def toggle_only_curr_workspace(self, *args):
      """Toggles the flag Only Current Workspace Windows in List"""
      if not self.no_toggling_signals:
         if self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == "True":
            self.gconf_client.set_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index, "False")
         else: self.gconf_client.set_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index, "True")
         self.store.load_model(self)
   
   def toggle_exit_after_tile_keyboard(self):
      """Toggles the flag Exit After Tile Through Shortcut"""
      if self.glade.configwindow.get_property("visible") == True:
         support.dialog_warning(_("The used shortcut is not available when the Preferences Window is opened"), self.glade.window)
      else: self.toggle_exit_after_tile()
   
   def toggle_only_curr_workspace_keyboard(self):
      """Toggles the flag Only Current Workspace Windows in List Through Shortcut"""
      if self.glade.configwindow.get_property("visible") == True:
         support.dialog_warning(_("The used shortcut is not available when the Preferences Window is opened"), self.glade.window)
      else: self.toggle_only_curr_workspace()
   
   def is_window_visible(self):
      """Returns True if the window is visible, False otherwise"""
      return self.glade.window.get_property("visible")
      
   def flag_all_rows(self, *args):
      """Flags All Rows"""
      self.store.flag_all_rows()
      
   def unflag_all_rows(self, *args):
      """Unflags All Rows"""
      self.store.unflag_all_rows()
      
   def make_pixbuf(self, treeviewcolumn, cell, model, iter):
      """Function to associate the pixbuf to the cell renderer"""
      pixbuf = model[iter][3]
      cell.set_property('pixbuf', pixbuf)
   
   def on_window_delete_event(self, widget, event, data=None):
      """Before close the application: no checks needed"""
      glob.alive = False
      self.quit_application()
      return True # do not propogate the delete event
      
   def on_configwindow_delete_event(self, widget, event, data=None):
      """Destroy the config window"""
      self.on_configwin_close_button_clicked()
      return True # do not propogate the delete event
      
   def dialog_preferences(self, *args):
      """Open the Config Window"""
      if self.glade.configwindow.get_property("visible") == True: return
      self.no_toggling_signals = True
      self.glade.exit_after_tile_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True")
      self.glade.current_workspace_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == "True")
      self.glade.show_toolbar_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index) == "True")
      self.glade.show_vbuttons_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_SHOW_VBUTTONS % glob.screen_index) == "True")
      self.no_toggling_signals = False
      self.glade.configwindow.show_all()
   
   def on_configwin_close_button_clicked(self, *args):
      """Hide the Dialog"""
      self.glade.configwindow.hide()
      cons.X1 = int(self.glade.spinbutton_x1.get_value())
      cons.Y1 = int(self.glade.spinbutton_y1.get_value())
      cons.W1 = int(self.glade.spinbutton_w1.get_value())
      cons.H1 = int(self.glade.spinbutton_h1.get_value())
      if cons.OVERRIDE_1 == 1:
         glob.monitors_areas[0] = [cons.X1, cons.Y1, cons.W1, cons.H1]
         self.gconf_client.set_int(cons.GCONF_X1 % glob.screen_index, cons.X1)
         self.gconf_client.set_int(cons.GCONF_Y1 % glob.screen_index, cons.Y1)
         self.gconf_client.set_int(cons.GCONF_W1 % glob.screen_index, cons.W1)
         self.gconf_client.set_int(cons.GCONF_H1 % glob.screen_index, cons.H1)
      if glob.num_monitors > 1:
         cons.X2 = int(self.glade.spinbutton_x2.get_value())
         cons.Y2 = int(self.glade.spinbutton_y2.get_value())
         cons.W2 = int(self.glade.spinbutton_w2.get_value())
         cons.H2 = int(self.glade.spinbutton_h2.get_value())
         if cons.OVERRIDE_2 == 1:
            glob.monitors_areas[1] = [cons.X2, cons.Y2, cons.W2, cons.H2]
            self.gconf_client.set_int(cons.GCONF_X2 % glob.screen_index, cons.X2)
            self.gconf_client.set_int(cons.GCONF_Y2 % glob.screen_index, cons.Y2)
            self.gconf_client.set_int(cons.GCONF_W2 % glob.screen_index, cons.W2)
            self.gconf_client.set_int(cons.GCONF_H2 % glob.screen_index, cons.H2)
   
   def on_button_row_up_clicked(self, *args):
      """Move the selected row up of one position"""
      model, iter = self.viewselection.get_selected()
      if iter == None:
         support.dialog_warning(_("No Row is Selected"), self.glade.window)
         return
      self.store.row_up(iter)
   
   def on_button_row_down_clicked(self, *args):
      """Move the selected row up of one position"""
      model, iter = self.viewselection.get_selected()
      if iter == None:
         support.dialog_warning(_("No Row is Selected"), self.glade.window)
         return
      self.store.row_down(iter)
   
   def on_button_row_delete_clicked(self, *args):
      """Remove the selected row"""
      model, iter = self.viewselection.get_selected()
      if iter == None:
         support.dialog_warning(_("No Row is Selected"), self.glade.window)
         return
      self.store.row_delete(iter)
   
   def on_button_add_filter_clicked(self, *args):
      """Application Filter Add"""
      self.filter_add_list_exist_or_create()
      self.filter_add_liststore.clear()
      for element in self.store.process_picklist:
         self.filter_add_liststore.append([element])
      response = self.glade.filteradddialog.run()
      self.glade.filteradddialog.hide()
      if response != 1: return
      model, iter = self.filter_add_treeviewselection.get_selected()
      if iter == None: return
      self.store.process_blacklist.add(model[iter][0])
      self.filter_list_update()
   
   def on_button_remove_filter_clicked(self, *args):
      """Application Filter Remove"""
      model, iter = self.filter_treeviewselection.get_selected()
      if iter == None:
         support.dialog_warning(_("No Application Selected!"), self.glade.window)
         return
      self.store.process_blacklist.remove(model[iter][0])
      self.filter_list_update()
   
   def dialog_filter(self, *args):
      """Applications Filter Dialog"""
      self.filter_list_exist_or_create()
      self.filter_list_update()
      self.glade.filterdialog.run()
      self.glade.filterdialog.hide()
      if len(self.store.process_blacklist) > 0:
         self.gconf_client.set_string(cons.GCONF_PROCESS_BLACKLIST % glob.screen_index, " ".join(self.store.process_blacklist))
      else: self.gconf_client.set_string(cons.GCONF_PROCESS_BLACKLIST % glob.screen_index, "")
      self.reload_windows_list()
   
   def filter_list_update(self):
      """Updates the Process Filter List"""
      self.filter_liststore.clear()
      for element in self.store.process_blacklist:
         self.filter_liststore.append([element])
   
   def filter_list_exist_or_create(self):
      """If The List Was Never Used, this will Create It"""
      if not "filter_liststore" in dir(self):
         self.filter_liststore = gtk.ListStore(str)
         self.filter_treeview = gtk.TreeView(self.filter_liststore)
         self.filter_treeview.set_headers_visible(False)
         self.filter_renderer_text = gtk.CellRendererText()
         self.filter_column = gtk.TreeViewColumn("Application", self.filter_renderer_text, text=0)
         self.filter_treeview.append_column(self.filter_column)
         self.filter_treeviewselection = self.filter_treeview.get_selection()
         self.glade.scrolledwindow_filter.add(self.filter_treeview)
         self.glade.scrolledwindow_filter.show_all()
   
   def filter_add_list_exist_or_create(self):
      """If The List Was Never Used, this will Create It"""
      if not "filter_add_liststore" in dir(self):
         self.filter_add_liststore = gtk.ListStore(str)
         self.filter_add_treeview = gtk.TreeView(self.filter_add_liststore)
         self.filter_add_treeview.set_headers_visible(False)
         self.filter_add_renderer_text = gtk.CellRendererText()
         self.filter_add_column = gtk.TreeViewColumn("Application", self.filter_add_renderer_text, text=0)
         self.filter_add_treeview.append_column(self.filter_add_column)
         self.filter_add_treeviewselection = self.filter_add_treeview.get_selection()
         self.filter_add_treeview.connect('button-press-event', self.on_mouse_button_clicked_filter_add)
         self.glade.scrolledwindow_filter_add.add(self.filter_add_treeview)
         self.glade.scrolledwindow_filter_add.show_all()
   
   def on_mouse_button_clicked_filter_add(self, widget, event):
      """Catches mouse buttons clicks"""
      if event.button != 1: return
      if event.type == gtk.gdk._2BUTTON_PRESS: self.glade.filteradddialog_button_ok.clicked()
   
   def on_key_press_filteradddialog(self, widget, event):
      """Catches AnchorHandle Dialog key presses"""
      keyname = gtk.gdk.keyval_name(event.keyval)
      if keyname == "Return": self.glade.filteradddialog_button_ok.clicked()
   
   def quit_application(self, *args):
      """Hide the window"""
      actual_win_size = list(self.glade.window.get_size())
      actual_win_pos = list(self.glade.window.get_position())
      if actual_win_size != self.win_size_n_pos['win_size']:
         self.win_size_n_pos['win_size'] = actual_win_size
         self.gconf_client.set_int(cons.GCONF_WIN_SIZE_W % glob.screen_index, self.win_size_n_pos['win_size'][0])
         self.gconf_client.set_int(cons.GCONF_WIN_SIZE_H % glob.screen_index, self.win_size_n_pos['win_size'][1])
      if actual_win_pos != self.win_size_n_pos['win_position']:
         self.win_size_n_pos['win_position'] = actual_win_pos
         self.gconf_client.set_int(cons.GCONF_WIN_POSITION_X % glob.screen_index, self.win_size_n_pos['win_position'][0])
         self.gconf_client.set_int(cons.GCONF_WIN_POSITION_Y % glob.screen_index, self.win_size_n_pos['win_position'][1])
      self.glade.window.destroy()
      if len(sys.argv) > 1 and sys.argv[1] == "--without-panel": gtk.main_quit()
   
   def launch_application(self):
      """Show the main window and all child widgets"""
      self.init_from_gconf()
      self.update_statusbar()
      self.glade.window.show_all()
      show_toolbar = self.gconf_client.get_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index)
      if show_toolbar == None: self.gconf_client.set_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index, "True")
      elif show_toolbar == "False": self.ui.get_widget("/ToolBar").hide()
      show_vbuttons = self.gconf_client.get_string(cons.GCONF_SHOW_VBUTTONS % glob.screen_index)
      if show_vbuttons == None: self.gconf_client.set_string(cons.GCONF_SHOW_VBUTTONS % glob.screen_index, "False")
      elif show_vbuttons == "False": self.glade.vbuttonbox_updown.hide()
      self.window_position_restore()
   
   def reload_windows_list(self, *args):
      """Reloads the Windows List"""
      self.store.load_model(self)
   
   def close_checked_windows(self, *args):
      """Closes the Checked Windows"""
      number_of_windows = len(self.store.get_checked_windows_list())
      if number_of_windows < 1: support.dialog_warning(_('No Windows Checked'), self.glade.window)
      else: self.store.close_checked_windows()
   
   def maximize_checked_windows(self, *args):
      """Maximizes the Checked Windows"""
      checked_windows_lists = self.store.get_checked_windows_list()
      checked_windows_list = checked_windows_lists[0] + checked_windows_lists[1]
      number_of_windows = len(checked_windows_list)
      if number_of_windows < 1: support.dialog_warning(_('No Windows Checked'), self.glade.window)
      else:
         for checked_window in checked_windows_list: support.maximize(checked_window)
         if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
            glob.alive = False
            self.quit_application()
   
   def unmaximize_checked_windows(self, *args):
      """Unmaximizes the Checked Windows"""
      checked_windows_lists = self.store.get_checked_windows_list()
      checked_windows_list = checked_windows_lists[0] + checked_windows_lists[1]
      number_of_windows = len(checked_windows_list)
      if number_of_windows < 1: support.dialog_warning(_('No Windows Checked'), self.glade.window)
      else:
         for checked_window in checked_windows_list: support.unmaximize(checked_window)
         if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
            glob.alive = False
            self.quit_application()
   
   def tile_vertically(self, *args):
      """Tile the Checked Windows Vertically"""
      checked_windows_list = self.store.get_checked_windows_list()
      tilings.tile_vertically(checked_windows_list, glob.monitors_areas)
      time.sleep(cons.TILING_BREAK)
      tilings.tile_vertically(checked_windows_list, glob.monitors_areas)
      if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
         glob.alive = False
         self.quit_application()
   
   def tile_horizontally(self, *args):
      """Tile the Checked Windows Horizontally"""
      checked_windows_list = self.store.get_checked_windows_list()
      tilings.tile_horizontally(checked_windows_list, glob.monitors_areas)
      time.sleep(cons.TILING_BREAK)
      tilings.tile_horizontally(checked_windows_list, glob.monitors_areas)
      if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
         glob.alive = False
         self.quit_application()
   
   def tile_quad(self, *args):
      """Tile the Checked Windows Quad"""
      checked_windows_list = self.store.get_checked_windows_list()
      tilings.tile_quad(checked_windows_list, glob.monitors_areas)
      time.sleep(cons.TILING_BREAK)
      tilings.tile_quad(checked_windows_list, glob.monitors_areas)
      if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
         glob.alive = False
         self.quit_application()
   
   def tile_triangle_up(self, *args):
      """Tile 3 Windows in Triangle Up Scheme"""
      checked_windows_list = self.store.get_checked_windows_list()
      tilings.tile_triangle_up(checked_windows_list, glob.monitors_areas)
      time.sleep(cons.TILING_BREAK)
      tilings.tile_triangle_up(checked_windows_list, glob.monitors_areas)
      if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
         glob.alive = False
         self.quit_application()
      
   def tile_triangle_down(self, *args):
      """Tile 3 Windows in Triangle Down Scheme"""
      checked_windows_list = self.store.get_checked_windows_list()
      tilings.tile_triangle_down(checked_windows_list, glob.monitors_areas)
      time.sleep(cons.TILING_BREAK)
      tilings.tile_triangle_down(checked_windows_list, glob.monitors_areas)
      if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
         glob.alive = False
         self.quit_application()
      
   def tile_triangle_left(self, *args):
      """Tile 3 Windows in Triangle Left Scheme"""
      checked_windows_list = self.store.get_checked_windows_list()
      tilings.tile_triangle_left(checked_windows_list, glob.monitors_areas)
      time.sleep(cons.TILING_BREAK)
      tilings.tile_triangle_left(checked_windows_list, glob.monitors_areas)
      if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
         glob.alive = False
         self.quit_application()
      
   def tile_triangle_right(self, *args):
      """Tile 3 Windows in Triangle Right Scheme"""
      checked_windows_list = self.store.get_checked_windows_list()
      tilings.tile_triangle_right(checked_windows_list, glob.monitors_areas)
      time.sleep(cons.TILING_BREAK)
      tilings.tile_triangle_right(checked_windows_list, glob.monitors_areas)
      if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == "True": 
         glob.alive = False
         self.quit_application()
   
   def show_hide_toolbar(self, menuitem, data=None):
      """Show/Hide the Toolbar"""
      if not self.no_toggling_signals:
         if self.ui.get_widget("/ToolBar").get_property('visible') == True:
            self.ui.get_widget("/ToolBar").hide()
            self.gconf_client.set_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index, "False")
         else:
            self.ui.get_widget("/ToolBar").show()
            self.gconf_client.set_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index, "True")
         
   def show_hide_vertical_buttons(self, menuitem, data=None):
      """Show/Hide the Toolbar"""
      if not self.no_toggling_signals:
         if self.glade.vbuttonbox_updown.get_property('visible') == True:
            self.glade.vbuttonbox_updown.hide()
            self.gconf_client.set_string(cons.GCONF_SHOW_VBUTTONS % glob.screen_index, "False")
         else:
            self.glade.vbuttonbox_updown.show()
            self.gconf_client.set_string(cons.GCONF_SHOW_VBUTTONS % glob.screen_index, "True")
   
   def dialog_about(self, menuitem, data=None):
      """Show the About Dialog and hide it when a button is pressed"""
      self.glade.aboutdialog.run()
      self.glade.aboutdialog.hide()
      
   def on_help_menu_item_activated(self, menuitem, data=None):
      """Show the application's Instructions"""
      webbrowser.open("http://open.vitaminap.it/en/x_tile.htm")


